//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Base/Element/IElement.h
//! @brief     Defines class DiffuseElement.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifdef SWIG
#error no need to expose this header to Swig
#endif // SWIG
#ifndef BORNAGAIN_BASE_ELEMENT_IELEMENT_H
#define BORNAGAIN_BASE_ELEMENT_IELEMENT_H

#include "Base/Element/PolMatrices.h"

#include <utility>

//! @ingroup simulation

class IElement {
public:
    explicit IElement(PolMatrices polMatrices)
        : m_polMatrices(std::move(polMatrices))
    {
    }

    //! Returns polarizer and analyzer matrices.
    const PolMatrices& polMatrices() const { return m_polMatrices; }

protected:
    const PolMatrices m_polMatrices;
};

#endif // BORNAGAIN_BASE_ELEMENT_IELEMENT_H
