//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Device/Coord/AxisNames.h
//! @brief     Defines axisUnitLabel and maps in namespace AxisNames.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifdef SWIG
#error no need to expose this header to Swig
#endif // SWIG
#ifndef BORNAGAIN_DEVICE_COORD_AXISNAMES_H
#define BORNAGAIN_DEVICE_COORD_AXISNAMES_H

#include "Device/Coord/Tags.h"
#include "Wrap/WinDllMacros.h"
#include <map>
#include <string>

const std::map<Coords, const char*> axisUnitLabel = {
    {Coords::UNDEFINED, "undefined"}, {Coords::NBINS, "bin"}, {Coords::RADIANS, "rad"},
    {Coords::DEGREES, "deg"},         {Coords::MM, "mm"},     {Coords::QSPACE, "1/nm"},
    {Coords::QXQY, "1/nm"},           {Coords::RQ4, "nm^-4?"}};

//! Contains const maps that give the right axis names for different detector types and units
//! @ingroup detector

namespace DataUtils::AxisNames {

extern BA_DEVICE_API_ const std::map<Coords, std::string> sphericalAxis0;
extern BA_DEVICE_API_ const std::map<Coords, std::string> sphericalAxis1;
extern BA_DEVICE_API_ const std::map<Coords, std::string> rectangularAxis0;
extern BA_DEVICE_API_ const std::map<Coords, std::string> rectangularAxis1;
extern BA_DEVICE_API_ const std::map<Coords, std::string> offspecAxis0;
extern BA_DEVICE_API_ const std::map<Coords, std::string> offspecAxis1;
extern BA_DEVICE_API_ const std::map<Coords, std::string> specAxis;
extern BA_DEVICE_API_ const std::map<Coords, std::string> specAxisQ;
extern BA_DEVICE_API_ const std::map<Coords, std::string> sampleDepthAxis;

} // namespace DataUtils::AxisNames

#endif // BORNAGAIN_DEVICE_COORD_AXISNAMES_H
