//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Sample/SampleModel.cpp
//! @brief     Implements class SampleModel
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Model/Sample/SampleModel.h"
#include "GUI/Model/Sample/ItemWithMaterial.h"
#include "GUI/Model/Sample/SampleItem.h"
#include "GUI/Support/XML/UtilXML.h"
#include <QApplication>

namespace {
namespace Tag {

const QString Sample("Sample");

} // namespace Tag
} // namespace

SampleModel::~SampleModel() {}

QVector<SampleItem*> SampleModel::sampleItems() const
{
    return QVector<SampleItem*>(m_samples.begin(), m_samples.end());
}

SampleItem* SampleModel::addSample()
{
    m_samples.emplace_back(new SampleItem());
    return m_samples.back();
}

void SampleModel::addSample(SampleItem* sample)
{
    m_samples.emplace_back(sample);
}

void SampleModel::removeSample(SampleItem* sample)
{
    m_samples.delete_element(sample);
}

QStringList SampleModel::sampleNames() const
{
    QStringList existingNames;
    for (const auto* item : m_samples)
        existingNames << item->sampleName();
    return existingNames;
}

void SampleModel::writeTo(QXmlStreamWriter* w) const
{
    XML::writeAttribute(w, XML::Attrib::version, uint(1));

    // samples
    for (const auto* sample : m_samples) {
        w->writeStartElement(Tag::Sample);
        sample->writeTo(w);
        w->writeEndElement();
    }
}

void SampleModel::readFrom(QXmlStreamReader* r)
{
    m_samples.clear();

    const uint version = XML::readUIntAttribute(r, XML::Attrib::version);
    Q_UNUSED(version)

    while (r->readNextStartElement()) {
        QString tag = r->name().toString();

        // sample
        if (tag == Tag::Sample) {
            addSample()->readFrom(r);
            XML::gotoEndElementOfTag(r, tag);

        } else
            r->skipCurrentElement();
    }
}
