//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Support/XML/UtilXML.h
//! @brief     Defines
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_SUPPORT_XML_UTILXML_H
#define BORNAGAIN_GUI_SUPPORT_XML_UTILXML_H

#include <QVariant>
#include <QXmlStreamWriter> // used in every including file
#include <heinz/Complex.h>
#include <heinz/Vectors3D.h>

namespace XML {
namespace Attrib {

const QString version("version");
const QString selection_version("selection_version");
const QString binary_version("binary_version");
const QString value("value");
const QString BA_Version("BA_Version");
const QString projectName("projectName");
const QString name("name");
const QString id("id");
const QString type("type");
const QString size("size");

} // namespace Attrib

namespace Tags {
constexpr auto TagAttribute("Tag");
constexpr auto ModelTypeAttribute("ModelType");
constexpr auto DisplayNameAttribute("DisplayName");
constexpr auto ParameterTag("Parameter");
constexpr auto BinaryData("BinaryData");
constexpr auto NonSessionItemData("NonSessionItemData");
constexpr auto ParameterNameAttribute("ParName");
constexpr auto ParameterTypeAttribute("ParType");
constexpr auto ParameterValueAttribute("ParValue");
constexpr auto ParameterRoleAttribute("ParRole");
constexpr auto ParameterExtAttribute("ParExt");
constexpr auto ParameterToolAttribute("ParTool");
constexpr auto ExternalPropertyColorAtt("Color");
constexpr auto ExternalPropertyIdentifierAtt("Identifier");
} // namespace Tags

constexpr auto Version("Version");

void gotoEndElementOfTag(QXmlStreamReader* reader, const QString& tag);

//! Write the variant as a complete tag, including the given role
void writeVariant(QXmlStreamWriter* writer, QVariant variant, int role);

//! Write the variant's value as an attribute
void writeAttribute(QXmlStreamWriter* writer, const QString& attributeName,
                    const QVariant& variant);

void writeAttribute(QXmlStreamWriter* writer, const QString& attributeName, int d);
void writeAttribute(QXmlStreamWriter* writer, const QString& attributeName, bool b);
void writeAttribute(QXmlStreamWriter* writer, const QString& attributeName, unsigned d);
void writeAttribute(QXmlStreamWriter* writer, const QString& attributeName, double d);
void writeAttribute(QXmlStreamWriter* writer, const QString& attributeBaseName, const R3& vec);
void writeAttribute(QXmlStreamWriter* writer, const QString& attributeBaseName, const complex_t& c);

bool readBoolAttribute(QXmlStreamReader* reader, const QString& attributeName);
unsigned readUIntAttribute(QXmlStreamReader* reader, const QString& attributeName);

void readAttribute(QXmlStreamReader* reader, const QString& attributeName, int* i);
void readAttribute(QXmlStreamReader* reader, const QString& attributeName, bool* b);
void readAttribute(QXmlStreamReader* reader, const QString& attributeName, double* d);
void readAttribute(QXmlStreamReader* reader, const QString& attributeName, unsigned* u);
void readAttribute(QXmlStreamReader* reader, const QString& attributeName, QString* s);
void readAttribute(QXmlStreamReader* reader, const QString& attributeName, QColor* c);
void readAttribute(QXmlStreamReader* reader, const QString& attributeBaseName, R3* vec);
void readAttribute(QXmlStreamReader* reader, const QString& attributeBaseName, complex_t* c);

void assertExpectedTag(QXmlStreamReader* reader, const QString& tag);

template <typename T>
void writeVector(QXmlStreamWriter* writer, const QString& tag, const QVector<T>& vec)
{
    for (int i = 0; i < vec.size(); i++) {
        writer->writeStartElement(tag);
        XML::writeAttribute(writer, "n" + QString::number(i), vec[i]);
        writer->writeEndElement();
    }
}
template <typename T>
void readVector(QXmlStreamReader* reader, const QString& tag, QVector<T>& vec)
{
    vec.clear();
    int i = 0;
    while (reader->readNextStartElement()) {
        if (reader->name() == tag) {
            vec.resize(i + 1);
            XML::readAttribute(reader, "n" + QString::number(i), &vec[i]);
            i++;
            XML::gotoEndElementOfTag(reader, tag);
        } else {
            reader->skipCurrentElement();
        }
    }
}

} // namespace XML

#endif // BORNAGAIN_GUI_SUPPORT_XML_UTILXML_H
